﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Collections.Specialized;
using Infragistics.Controls.Maps;

namespace NetworkNode_BrindNodeIntoView
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        SampleTestData asd = new SampleTestData();
        public MainWindow()
        {
            InitializeComponent();
            DataContext = asd;
        }

        private void listBox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            xnn.SelectedNodes.Clear();    
            NetworkNodeNode node = xnn.Search((NodeModel item) => item.Label.StartsWith((listBox1.SelectedItem as NodeModel).Label)).FirstOrDefault();
            xnn.SelectedNodes.Add(node);           

            bool adjust = false;
            double targetLeft = xnn.WindowRect.Left;
            double targetRight = xnn.WindowRect.Right;
            double targetTop = xnn.WindowRect.Top;
            double targetBottom = xnn.WindowRect.Bottom;

            if (node.Location.X < xnn.WindowRect.Left)
            {
                targetLeft = node.Location.X;
                adjust = true;
            }

            if (node.Location.X + 120 > xnn.WindowRect.Right)
            {
                targetRight = node.Location.X + 120;
                adjust = true;
            }

            if (node.Location.Y < xnn.WindowRect.Top)
            {
                targetTop = node.Location.Y;
                adjust = true;
            }

            if (node.Location.Y + 40 > xnn.WindowRect.Bottom)
            {
                targetBottom = node.Location.Y + 120;
                adjust = true;
            }

            if (adjust)
            {
                xnn.WindowRect = new Rect(new Point(targetLeft, targetTop), new Point(targetRight, targetBottom));
                //xnn.InvalidateVisual();
                //xnn.InvalidateArrange();
            }

        }        
    }

    public class SampleTestData
    {
        public ObservableCollection<NodeModel> Nodes { get; set; }

        private const int K = 3;            // number of connections per node (maximum)
        private const int NUM_NODES = 98;   // number of nodes in the graph

        public SampleTestData()
        {
            Nodes = new ObservableCollection<NodeModel>();

            // Add NUM_NODES node objects to the collection
            for (int i = 0; i < NUM_NODES; i++)
            {
                NodeModel node = new NodeModel();
                node.Label = i.ToString();
                node.ToolTip = "ToolTip for " + node.Label;
                Nodes.Add(node);
            }

            int root = 0;
            int first = 1;
            int last = K;
            while (first < Nodes.Count)
            {
                Nodes[root].Connections = new ObservableCollection<NetworkConnection>();
                for (int i = first; i <= last; i++)
                {
                    if (i >= Nodes.Count)
                    {
                        break;
                    }
                    Nodes[root].Connections.Add(new NetworkConnection { Target = Nodes[i] });
                }
                root++;
                first = last + 1;
                last += K;
            }
        }
    }

    public class NodeModel : ObservableModel
    {
        private string _label;
        public string Label
        {
            get { return _label; }
            set
            {
                if (value != _label)
                {
                    _label = value;
                    OnPropertyChanged("Label");
                }
            }
        }

        private string _toolTip;
        public string ToolTip
        {
            get { return _toolTip; }
            set
            {
                if (value != _toolTip)
                {
                    _toolTip = value;
                    OnPropertyChanged("ToolTip");
                }
            }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
            set
            {
                if (value != _description)
                {
                    _description = value;
                    OnPropertyChanged("Description");
                }
            }
        }

        private string _imagePath;
        public string ImagePath
        {
            get { return _imagePath; }
            set
            {
                if (value != _imagePath)
                {
                    _imagePath = value;
                    OnPropertyChanged("ImagePath");
                }
            }
        }

        public enum NodeModelType { Default, Attendee, Speaker, Company, Convention };
        private NodeModelType _nodeType;
        public NodeModelType NodeType
        {
            get { return _nodeType; }
            set
            {
                if (value != _nodeType)
                {
                    _nodeType = value;
                    OnPropertyChanged("NodeType");
                }
            }
        }

        private ObservableCollection<NetworkConnection> _connections;
        public ObservableCollection<NetworkConnection> Connections
        {
            get { return _connections; }
            set
            {
                if (value != _connections)
                {
                    _connections = value;
                    OnPropertyChanged("Connections");
                }
            }
        }

    }

    public class NetworkConnection : ObservableModel
    {
        private int _weight;
        public int Weight
        {
            get { return _weight; }
            set
            {
                if (value != _weight)
                {
                    _weight = value;
                    OnPropertyChanged("Weight");
                }
            }
        }

        private NodeModel _target;
        public NodeModel Target
        {
            get { return _target; }
            set
            {
                if (value != _target)
                {
                    _target = value;
                    OnPropertyChanged("Target");
                }
            }
        }

    }

    public class ObservableModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
 
}
